/// <reference types="node" />
import express from 'express';
import * as http from 'http';
import * as t from 'runtypes';
import { Configuration } from 'botbuilder-runtime';
import { IServices, ServiceCollection } from 'botbuilder-runtime-core';
declare const TypedOptions: t.Record<{
    /**
     * Port that server should listen on
     */
    port: t.Number;
    /**
     * Path that the server will listen to for [Activities](xref:botframework-schema.Activity)
     */
    messagingEndpointPath: t.String;
}, false>;
/**
 * Options for runtime Express adapter
 */
export declare type Options = t.Static<typeof TypedOptions>;
/**
 * Start a bot using the runtime Express integration.
 *
 * @param applicationRoot application root directory
 * @param settingsDirectory settings directory
 * @param options options bag
 */
export declare function start(applicationRoot: string, settingsDirectory: string, options?: Partial<Options>): Promise<void>;
/**
 * Create an Express App using the runtime Express integration.
 *
 * @param services runtime service collection
 * @param configuration runtime configuration
 * @param options options bag for configuring Express Application
 * @returns the Express Application and a function to start the App & handle "upgrade" requests for Streaming
 */
export declare function makeApp(services: ServiceCollection<IServices>, configuration: Configuration, options?: Partial<Options>): Promise<[app: express.Application, listen: (callback?: () => void) => http.Server]>;
export {};
//# sourceMappingURL=index.d.ts.map